/* $Header: /CVSROOT/debris/drisnext.h,v 1.2 2005/06/01 02:39:04 tino Exp $
 *
 * DebRIS: DEBian Remote Install Scripter
 * Fetch the "next" step from the network (or whatever).
 *
 * $Log: drisnext.h,v $
 * Revision 1.2  2005/06/01 02:39:04  tino
 * now it compiles
 *
 * Revision 1.1  2005/06/01 02:12:35  tino
 * first version.  As SirbeD is missing, this just does nothing yet.
 *
 */

#include "driscurl.h"

#include <openssl/md5.h>

static MD5_CTX	dris_cksum;

static void
dris_cksum_add(const void *ptr, size_t len)
{
  if (len)
    MD5_Update(&dris_cksum, ptr, len);
}

static void
dris_cksum_add_s(const char *s)
{
  dris_cksum_add(s, strlen(s));
}

static void
dris_cksum_init(void)
{
  MD5_Init(&dris_cksum);
  dris_cksum_add_s(config.pw);
}

static const char *
dris_cksum_get(void)
{
  static const char	hex[]	= "0123456789abcdef";
  static char		ret[40];
  char			digest[16], *ptr;
  int			i;

  MD5_Final(digest, &dris_cksum);

  ptr	= ret;
  for (i=0; i<16; i++)
    {
      *ptr++	= hex[(digest[i]>>4)&0xf];
      *ptr++	= hex[(digest[i]>>0)&0xf];
    }
  *ptr++	= '\r';
  *ptr++	= '\n';
  *ptr		= 0;
  return ret;
}

static const char *
dris_next_check(struct dris_buf *buf)
{
  int		i;
  const char	*cmp;

  for (i=0; buf->buf[i]!='\r' || buf->buf[i+1]!='\n'; i++)
    if (!buf->buf[i])
      return "missing first line terminator";
  if ((i+=2)>buf->len)
    return "BUG: i>buf->len";
  dris_cksum_init();
  dris_cksum_add(buf->buf+i, buf->len-i);
  cmp	= dris_cksum_get();
  if (strncmp(buf->buf, cmp, strlen(cmp)))
    return "checksum mismatch";
  dris_buf_add(dris_buf_reset(config.out), buf->buf+i, buf->len-i);
  return 0;
}

/* This here is th black magic function to fetch the next step to do.
 *
 * It knows about the config struct.
 * It contacts URL for data transfer.
 * It sends config.result and config.out to the network.
 * It reads config.in from the network.
 * It verifies the correctness of the received data.
 */
static int
dris_next(void)
{
  static struct dris_buf	*in, *out;
  static int (			*fn)(struct dris_buf *, struct dris_buf *) = dris_curl;
  int				retry;
  char				tmp[256];

  if (!in)
    in	= dris_buf_new();
  if (!out)
    out	= dris_buf_new();

  sprintf(tmp, "%s %lld\r\n%d\r\n", config.id, config.cnt, config.result);

  dris_cksum_init();
  dris_cksum_add_s(tmp);
  dris_cksum_add(config.out->buf, config.out->len);

  dris_buf_reset(in);
  dris_buf_add_s(in, dris_cksum_get());
  dris_buf_add_s(in, tmp);
  dris_buf_add(in, config.out->buf, config.out->len);

  retry	= 0;
  for (;;)
    {
      int	err;

      dris_buf_reset(out);
      verbose("[%lld] send %ld bytes to %s\n", config.cnt, (long)in->len, config.url);
      if ((err=fn(in, out))==0)
	{
	  const char	*err;

	  verbose("* got %lu bytes\n", (unsigned long)out->len);
	  err	= dris_next_check(out);
	  if (!err)
	    return 1;
	  verbose("* data error: %s\n", err);
	}
      retry++;
      verbose("* failed with error code %d, sleeping ", err, retry);
      sleep(retry>60 ? 60 : retry);
      verbose(".. retry %d\n", retry);
    }
}
